# -*- coding: utf-8 -*-

from unittest.mock import patch

from django.test import TestCase, override_settings

from rest_search import connections


class ConnectionTest(TestCase):
    def tearDown(self):
        del connections["default"]

    @override_settings(
        REST_SEARCH_CONNECTIONS={
            "default": {
                "AWS_ACCESS_KEY": "some-access-key",
                "AWS_REGION": "some-region",
                "AWS_SECRET_KEY": "some-secret-key",
                "HOST": "es.example.com",
            }
        }
    )
    @patch("rest_search.OpenSearch")
    def test_aws_auth(self, mock_opensearch):
        es = connections["default"]
        self.assertIsNotNone(es)

        self.assertEqual(mock_opensearch.call_count, 1)
        self.assertEqual(mock_opensearch.call_args[0], ())
        self.assertEqual(
            sorted(mock_opensearch.call_args[1].keys()),
            ["connection_class", "host", "http_auth", "port", "use_ssl"],
        )

    @override_settings(
        REST_SEARCH_CONNECTIONS={
            "default": {
                "AWS_REGION": "some-region",
                "HOST": "es.example.com",
            }
        }
    )
    @patch("rest_search.OpenSearch")
    def test_aws_role_auth(self, mock_opensearch):
        es = connections["default"]
        self.assertIsNotNone(es)

        self.assertEqual(mock_opensearch.call_count, 1)
        self.assertEqual(mock_opensearch.call_args[0], ())
        self.assertEqual(
            sorted(mock_opensearch.call_args[1].keys()),
            ["connection_class", "host", "http_auth", "port", "use_ssl"],
        )

    @patch("rest_search.OpenSearch")
    def test_no_auth(self, mock_opensearch):
        es = connections["default"]
        self.assertIsNotNone(es)

        self.assertEqual(mock_opensearch.call_count, 1)
        self.assertEqual(mock_opensearch.call_args[0], ())
        self.assertEqual(
            sorted(mock_opensearch.call_args[1].keys()), ["host", "port", "use_ssl"]
        )



# -*- coding: utf-8 -*-

from unittest.mock import patch

from django.test import TestCase


class ViewsTest(TestCase):
    @patch("rest_search.tasks.patch_index.delay")
    def test_create(self, mock_delay):
        response = self.client.post("/books", {"title": "New book"})
        self.assertEqual(response.status_code, 201)

    @patch("opensearchpy.client.OpenSearch.search")
    def test_search(self, mock_search):
        mock_search.return_value = {
            "_shards": {"failed": 0, "successful": 5, "total": 5},
            "hits": {
                "hits": [
                    {
                        "_id": "1",
                        "_index": "book",
                        "_score": 1,
                        "_source": {
                            "id": 1,
                            "tags": ["foo", "bar"],
                            "title": "New book",
                        },
                    }
                ],
                "max_score": 0.60911113,
                "total": {"value": 1, "relation": "eq"},
            },
            "timed_out": False,
            "took": 39,
        }

        response = self.client.get("/books/search", {"title": "New book"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 1,
                "next": None,
                "previous": None,
                "results": [{"tags": ["foo", "bar"], "title": "New book"}],
            },
        )

        mock_search.assert_called_once_with(
            body={
                "from": 0,
                "query": {"match_all": {}},
                "size": 20,
                "track_total_hits": True,
            },
            index="book",
        )

    def test_search_invalid(self):
        response = self.client.get("/books/search", {"id": "a"})
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.data, {"id": ["Enter a whole number."]})

    @patch("opensearchpy.client.OpenSearch.search")
    def test_search_pagination(self, mock_search):
        mock_search.return_value = {
            "_shards": {"failed": 0, "successful": 5, "total": 5},
            "hits": {
                "hits": [],
                "max_score": 0,
                "total": {"value": 1, "relation": "eq"},
            },
            "timed_out": False,
            "took": 24,
        }

        response = self.client.get(
            "/books/search", {"limit": 10, "offset": 1, "title": "New book"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 1,
                "next": None,
                "previous": "http://testserver/books/search?limit=10&title=New+book",
                "results": [],
            },
        )

        mock_search.assert_called_once_with(
            body={
                "from": 1,
                "query": {"match_all": {}},
                "size": 10,
                "track_total_hits": True,
            },
            index="book",
        )

    @patch("opensearchpy.client.OpenSearch.search")
    def test_search_sorted(self, mock_search):
        mock_search.return_value = {
            "_shards": {"failed": 0, "successful": 5, "total": 5},
            "hits": {
                "hits": [
                    {
                        "_id": "1",
                        "_index": "book",
                        "_score": 1,
                        "_source": {
                            "id": 1,
                            "tags": ["foo", "bar"],
                            "title": "New book",
                        },
                    }
                ],
                "max_score": 0.60911113,
                "total": {"value": 1, "relation": "eq"},
            },
            "timed_out": False,
            "took": 39,
        }

        response = self.client.get("/books/search_sorted", {"title": "New book"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 1,
                "next": None,
                "previous": None,
                "results": [{"tags": ["foo", "bar"], "title": "New book"}],
            },
        )

        mock_search.assert_called_once_with(
            body={
                "from": 0,
                "query": {"match_all": {}},
                "size": 20,
                "sort": [{"id": {"order": "desc"}}],
                "track_total_hits": True,
            },
            index="book",
        )



# -*- coding: utf-8 -*-

import uuid
from unittest.mock import patch

from django.test import TestCase

from rest_search.decorators import flush_updates
from tests.models import Author, Book


class DecoratorsTest(TestCase):
    @patch("rest_search.tasks.patch_index.delay")
    def test_flush_updates_author(self, mock_delay):
        @flush_updates
        def create_author():
            return Author.objects.create(
                name="Some author", pk=uuid.UUID("a5df7423-2ed3-452b-b13e-c643d3f4e85d")
            )

        create_author()
        mock_delay.assert_called_with(
            {"Author": ["a5df7423-2ed3-452b-b13e-c643d3f4e85d"]}
        )

    @patch("rest_search.tasks.patch_index.delay")
    def test_flush_updates_book(self, mock_delay):
        @flush_updates
        def create_book():
            return Book.objects.create(pk=1, title="Some book")

        create_book()
        mock_delay.assert_called_with({"Book": ["1"]})



# -*- coding: utf-8 -*-

from django import forms
from django.test import TestCase
from rest_framework.schemas import openapi

from rest_search.forms import SearchForm
from rest_search.schemas import (
    get_form_field_openapi_schema,
    get_form_schema_operation_parameters,
)
from tests.forms import BookSearchForm


class FormFieldOpenapiSchemaTest(TestCase):
    def test_boolean(self):
        form_field = forms.BooleanField()
        self.assertEqual(get_form_field_openapi_schema(form_field), {"type": "boolean"})

    def test_date(self):
        form_field = forms.DateField()
        self.assertEqual(
            get_form_field_openapi_schema(form_field),
            {"type": "string", "format": "date"},
        )

    def test_datetime(self):
        form_field = forms.DateTimeField()
        self.assertEqual(
            get_form_field_openapi_schema(form_field),
            {"type": "string", "format": "date-time"},
        )

    def test_char(self):
        form_field = forms.CharField()
        self.assertEqual(get_form_field_openapi_schema(form_field), {"type": "string"})

    def test_float(self):
        form_field = forms.FloatField()
        self.assertEqual(get_form_field_openapi_schema(form_field), {"type": "number"})

    def test_integer(self):
        form_field = forms.IntegerField()
        self.assertEqual(get_form_field_openapi_schema(form_field), {"type": "integer"})


class FormSchemaTest(TestCase):
    def test_base_openapi(self):
        fields = get_form_schema_operation_parameters(SearchForm)
        self.assertEqual(fields, [])

    def test_book_openapi(self):
        fields = get_form_schema_operation_parameters(BookSearchForm)
        self.assertEqual(
            fields,
            [
                {
                    "name": "id",
                    "required": False,
                    "in": "query",
                    "description": "",
                    "schema": {"type": "integer"},
                },
                {
                    "name": "query",
                    "required": False,
                    "in": "query",
                    "description": "A full-text search on the name.",
                    "schema": {"type": "string"},
                },
                {
                    "name": "tags",
                    "required": False,
                    "in": "query",
                    "description": "",
                    "schema": {"type": "string"},
                },
            ],
        )


class SchemaGeneratorTest(TestCase):
    def test_openapi(self):
        generator = openapi.SchemaGenerator()
        schema = generator.get_schema()
        self.assertTrue("/books/search" in schema["paths"])

        parameters = schema["paths"]["/books/search"]["get"]["parameters"]
        self.assertEqual(len(parameters), 3)
        self.assertEqual(parameters[0]["name"], "id")
        self.assertEqual(parameters[1]["name"], "query")
        self.assertEqual(parameters[2]["name"], "tags")



import time

from celery import Celery
from django.test import TestCase, override_settings
from rest_framework.test import APIClient

from rest_search.decorators import flush_updates
from rest_search.tasks import create_index, delete_index
from tests.models import Tag

app = Celery()
app.config_from_object("tests.celeryconfig")


@override_settings(
    REST_SEARCH_CONNECTIONS={
        "default": {
            "HOST": "localhost",
        }
    },
)
class EndToEndTest(TestCase):
    client_class = APIClient

    def setUp(self):
        create_index()

        Tag.objects.create(slug="foo")
        Tag.objects.create(slug="bar")
        Tag.objects.create(slug="qux")

    def tearDown(self):
        delete_index()

    @flush_updates
    def test_book(self):
        # Create books.
        response = self.client.post(
            "/books", {"tags": ["foo", "bar"], "title": "Some book"}
        )
        self.assertEqual(response.status_code, 201)
        book_pk = response.data["id"]

        response = self.client.post("/books", {"tags": ["qux"], "title": "Other book"})
        self.assertEqual(response.status_code, 201)

        # Allow time for indexing.
        time.sleep(1)

        # Search books.
        response = self.client.get("/books/search")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 2,
                "next": None,
                "previous": None,
                "results": [
                    {"tags": ["foo", "bar"], "title": "Some book"},
                    {"tags": ["qux"], "title": "Other book"},
                ],
            },
        )

        response = self.client.get("/books/search", {"tags": "foo"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 1,
                "next": None,
                "previous": None,
                "results": [{"tags": ["foo", "bar"], "title": "Some book"}],
            },
        )

        # Update book.
        response = self.client.put(
            f"/books/{book_pk}", {"tags": ["foo"], "title": "Some book (changed)"}
        )
        self.assertEqual(response.status_code, 200)

        # Allow time for indexing.
        time.sleep(1)

        # Search books.
        response = self.client.get("/books/search")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 2,
                "next": None,
                "previous": None,
                "results": [
                    {"tags": ["qux"], "title": "Other book"},
                    {"tags": ["foo"], "title": "Some book (changed)"},
                ],
            },
        )

        # Delete book.
        response = self.client.delete(f"/books/{book_pk}")
        self.assertEqual(response.status_code, 204)

        # Allow time for indexing.
        time.sleep(1)

        # Search books.
        response = self.client.get("/books/search")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.data,
            {
                "count": 1,
                "next": None,
                "previous": None,
                "results": [
                    {"tags": ["qux"], "title": "Other book"},
                ],
            },
        )



# -*- coding: utf-8 -*-

import uuid
from unittest import mock
from unittest.mock import patch

from django.test import TestCase

from rest_search.tasks import create_index, delete_index, patch_index, update_index
from tests.models import Author, Book, Tag


class MockBulk(object):
    def __init__(self):
        self.actions = []

    def __call__(self, es, actions, raise_on_error=True):
        self.actions.extend(actions)


class TasksTest(TestCase):
    maxDiff = None

    def create_authors(self):
        Author.objects.create(
            name="Some author",
            unique_id=uuid.UUID("4996decc-c8f0-4492-800a-023ee7aaeec5"),
        )

    def create_books(self):
        book = Book.objects.create(id=1, title="Some book")
        book.tags.set([Tag.objects.create(slug=slug) for slug in ["foo", "bar"]])

    @patch("opensearchpy.client.indices.IndicesClient.exists")
    @patch("opensearchpy.client.indices.IndicesClient.create")
    def test_create_index(self, mock_create, mock_exists):
        mock_exists.return_value = False

        create_index()

        self.assertEqual(
            mock_exists.mock_calls,
            [
                mock.call("author"),
                mock.call("book"),
            ],
        )
        self.assertEqual(
            mock_create.mock_calls,
            [
                mock.call(
                    index="author",
                    body={
                        "mappings": {},
                        "settings": {
                            "analysis": {
                                "analyzer": {
                                    "default": {
                                        "tokenizer": "standard",
                                        "filter": ["lowercase", "asciifolding"],
                                    }
                                }
                            }
                        },
                    },
                ),
                mock.call(
                    index="book",
                    body={
                        "mappings": {"properties": {"tags": {"type": "keyword"}}},
                        "settings": {
                            "analysis": {
                                "analyzer": {
                                    "default": {
                                        "tokenizer": "standard",
                                        "filter": ["lowercase", "asciifolding"],
                                    }
                                }
                            }
                        },
                    },
                ),
            ],
        )

    @patch("opensearchpy.client.indices.IndicesClient.exists")
    @patch("opensearchpy.client.indices.IndicesClient.create")
    def test_create_index_exists(self, mock_create, mock_exists):
        mock_exists.return_value = True

        create_index()

        self.assertEqual(
            mock_exists.mock_calls,
            [
                mock.call("author"),
                mock.call("book"),
            ],
        )
        mock_create.assert_not_called()

    @patch("opensearchpy.client.indices.IndicesClient.delete")
    def test_delete_index(self, mock_delete):
        delete_index()

        self.assertEqual(
            mock_delete.mock_calls,
            [
                mock.call(index="author", ignore=404),
                mock.call(index="book", ignore=404),
            ],
        )

    @patch("rest_search.tasks.bulk")
    def test_patch_index(self, mock_bulk):
        self.create_authors()
        self.create_books()

        mock_bulk.side_effect = MockBulk()

        patch_index(
            {
                "Author": ["4996decc-c8f0-4492-800a-023ee7aaeec5"],
                "Book": ["1"],
            }
        )

        self.assertEqual(
            mock_bulk.side_effect.actions,
            [
                {
                    "_id": "4996decc-c8f0-4492-800a-023ee7aaeec5",
                    "_index": "author",
                    "_source": {
                        "name": "Some author",
                        "unique_id": "4996decc-c8f0-4492-800a-023ee7aaeec5",
                    },
                },
                {
                    "_id": "1",
                    "_index": "book",
                    "_source": {"id": 1, "tags": ["foo", "bar"], "title": "Some book"},
                },
            ],
        )

    @patch("rest_search.tasks.bulk")
    def test_patch_index_only_delete(self, mock_bulk):
        self.create_authors()
        self.create_books()

        mock_bulk.side_effect = MockBulk()

        patch_index({"Book": ["2"]})
        self.assertEqual(
            mock_bulk.side_effect.actions,
            [{"_id": "2", "_index": "book", "_op_type": "delete"}],
        )

    @patch("rest_search.tasks.bulk")
    def test_patch_index_with_delete(self, mock_bulk):
        self.create_authors()
        self.create_books()

        mock_bulk.side_effect = MockBulk()

        patch_index({"Book": ["1", "2"]})

        self.assertEqual(
            mock_bulk.side_effect.actions,
            [
                {
                    "_id": "1",
                    "_index": "book",
                    "_source": {"id": 1, "tags": ["foo", "bar"], "title": "Some book"},
                },
                {"_id": "2", "_index": "book", "_op_type": "delete"},
            ],
        )

    @patch("rest_search.tasks.create_index")
    @patch("rest_search.tasks.bulk")
    def test_update_index(self, mock_bulk, mock_create_index):
        self.create_books()

        mock_bulk.side_effect = MockBulk()

        def mock_scan(_es, *, index, **kwargs):
            # nothing in index
            return []

        with patch("rest_search.indexers.scan", new=mock_scan):
            update_index()

        mock_create_index.assert_called_once_with()
        self.assertEqual(
            mock_bulk.side_effect.actions,
            [
                {
                    "_id": "1",
                    "_index": "book",
                    "_source": {"id": 1, "tags": ["foo", "bar"], "title": "Some book"},
                }
            ],
        )

    @patch("rest_search.tasks.create_index")
    @patch("rest_search.tasks.bulk")
    def test_update_index_no_delete(self, mock_bulk, mock_create_index):
        self.create_books()

        mock_bulk.side_effect = MockBulk()

        def mock_scan(_es, *, index, **kwargs):
            return {
                "author": [],
                "book": [
                    {"_id": "1", "_index": "book", "_score": 0.0},  # still in DB
                    {"_id": "1001", "_index": "book", "_score": 0.0},  # gone from DB
                ],
            }[index]

        with patch("rest_search.indexers.scan", new=mock_scan):
            update_index(remove=False)

        mock_create_index.assert_called_once_with()
        self.assertEqual(
            mock_bulk.side_effect.actions,
            [
                {
                    "_id": "1",
                    "_index": "book",
                    "_source": {"id": 1, "tags": ["foo", "bar"], "title": "Some book"},
                }
            ],
        )

    @patch("rest_search.tasks.create_index")
    @patch("rest_search.tasks.bulk")
    def test_update_index_with_deleted_author(self, mock_bulk, mock_create_index):
        self.create_authors()

        mock_bulk.side_effect = MockBulk()

        def mock_scan(_es, *, index, **kwargs):
            return {
                "author": [
                    {
                        "_id": "4996decc-c8f0-4492-800a-023ee7aaeec5",
                        "_index": "author",
                        "_source": {
                            "name": "Some author",
                            "unique_id": "4996decc-c8f0-4492-800a-023ee7aaeec5",
                        },
                    },  # still in DB
                    {
                        "_id": "8550d60b-a3a7-4f07-8587-0b1a05a97e65",
                        "_index": "author",
                        "_source": {
                            "name": "Some author",
                            "unique_id": "8550d60b-a3a7-4f07-8587-0b1a05a97e65",
                        },
                    },  # gone from DB
                ],
                "book": [],
            }[index]

        with patch("rest_search.indexers.scan", new=mock_scan):
            update_index()

        mock_create_index.assert_called_once_with()
        self.assertEqual(
            mock_bulk.side_effect.actions,
            [
                {
                    "_id": "4996decc-c8f0-4492-800a-023ee7aaeec5",
                    "_index": "author",
                    "_source": {
                        "name": "Some author",
                        "unique_id": "4996decc-c8f0-4492-800a-023ee7aaeec5",
                    },
                },
                {
                    "_id": "8550d60b-a3a7-4f07-8587-0b1a05a97e65",
                    "_index": "author",
                    "_op_type": "delete",
                },
            ],
        )

    @patch("rest_search.tasks.create_index")
    @patch("rest_search.tasks.bulk")
    def test_update_index_with_deleted_book(self, mock_bulk, mock_create_index):
        self.create_books()

        mock_bulk.side_effect = MockBulk()

        def mock_scan(_es, *, index, **kwargs):
            return {
                "author": [],
                "book": [
                    {"_id": "1", "_index": "book", "_score": 0.0},  # still in DB
                    {"_id": "1001", "_index": "book", "_score": 0.0},  # gone from DB
                ],
            }[index]

        with patch("rest_search.indexers.scan", new=mock_scan):
            update_index()

        mock_create_index.assert_called_once_with()
        self.assertEqual(
            mock_bulk.side_effect.actions,
            [
                {
                    "_id": "1",
                    "_index": "book",
                    "_source": {"id": 1, "tags": ["foo", "bar"], "title": "Some book"},
                },
                {"_id": "1001", "_index": "book", "_op_type": "delete"},
            ],
        )



# -*- coding: utf-8 -*-

import uuid

from django.db import models
from django.db.models.signals import post_delete, post_save
from opensearchpy.helpers import scan

from rest_search import get_opensearch

_REGISTERED_CLASSES = []


class Indexer(object):
    mappings = None
    private_properties = []

    def __init__(self):
        model = self.serializer_class.Meta.model
        self.doc_type = model.__name__
        if not hasattr(self, "index"):
            self.index = model.__name__.lower()

        # Find the model's primary key.
        primary_key = model._meta.pk

        # Make a note of the field name.
        self.pk_name = primary_key.name

        # Determine how an `_id` from OpenSearch is parsed to
        # a primary key value.
        if isinstance(primary_key, models.UUIDField):
            self.pk_from_string = uuid.UUID
        elif isinstance(primary_key, (models.AutoField, models.BigAutoField)):
            self.pk_from_string = int
        else:
            raise AssertionError("Unhandled primary key type %s" % type(primary_key))

    def map_results(self, results):
        """
        Removes ES-specific fields from results.
        """

        def map_result_item(x):
            item = x["_source"]
            for key in self.private_properties:
                item.pop(key, None)
            return item

        return map(map_result_item, results)

    def scan(self, **kwargs):
        es = get_opensearch(self)
        return scan(es, index=self.index, **kwargs)

    def search(self, **kwargs):
        es = get_opensearch(self)
        return es.search(index=self.index, **kwargs)


def _get_registered():
    """
    Returns instances of all registered indexers.
    """
    return [cls() for cls in _REGISTERED_CLASSES]


def _instance_changed(sender, instance, **kwargs):
    """
    Queues an update to the OpenSearch index.
    """
    from rest_search import queue_add

    doc_type = instance.__class__.__name__
    queue_add({doc_type: [instance.pk]})


def register(indexer_class):
    """
    Registers an indexer class.
    """
    if indexer_class not in _REGISTERED_CLASSES:
        _REGISTERED_CLASSES.append(indexer_class)

        # register signal handlers
        model = indexer_class.serializer_class.Meta.model
        post_delete.connect(_instance_changed, sender=model)
        post_save.connect(_instance_changed, sender=model)



# -*- coding: utf-8 -*-

from unittest.mock import patch

from django.test import TestCase

from tests.indexers import BookIndexer, UnsupportedIndexer
from tests.models import Book, Tag


class IndexersTest(TestCase):
    def setUp(self):
        tags = [Tag.objects.create(slug=slug) for slug in ["foo", "bar"]]
        book = Book.objects.create(id=1, title="Some book")
        book.tags.set(tags)

    def test_map_results(self):
        indexer = BookIndexer()
        data = list(
            indexer.map_results(
                [
                    {
                        "_id": "1",
                        "_index": "book",
                        "_score": 0.0,
                        "_source": {
                            "id": 1,
                            "tags": ["foo", "bar"],
                            "title": "Some book",
                        },
                        "_type": "Book",
                    },
                    {
                        "_id": "2",
                        "_index": "book",
                        "_score": 0.0,
                        "_source": {"id": 2, "tags": [], "title": "Other book"},
                        "_type": "Book",
                    },
                ]
            )
        )
        self.assertEqual(
            data,
            [
                {"tags": ["foo", "bar"], "title": "Some book"},
                {"tags": [], "title": "Other book"},
            ],
        )

    @patch("rest_search.indexers.scan")
    def test_scan(self, mock_scan):
        mock_scan.return_value = [
            {"_id": "1", "_index": "book", "_score": 0.0, "_type": "Book"}
        ]

        indexer = BookIndexer()
        indexer.scan(body={"query": {"match_all": {}}})

    @patch("opensearchpy.client.OpenSearch.search")
    def test_search(self, mock_search):
        mock_search.return_value = {
            "_shards": {"failed": 0, "successful": 5, "total": 5},
            "hits": {"hits": [], "max_score": None, "total": 0},
            "timed_out": False,
            "took": 6,
        }

        indexer = BookIndexer()
        indexer.search(body={"query": {"match_all": {}}})

    def test_unsupported_primary_key(self):
        with self.assertRaises(AssertionError) as cm:
            UnsupportedIndexer()
        self.assertEqual(
            str(cm.exception),
            "Unhandled primary key type <class 'django.db.models.fields.URLField'>",
        )



# -*- coding: utf-8 -*-

from django import forms
from django.utils.encoding import force_str


def get_form_schema_operation_parameters(form_class):
    """
    Return the openapi schema for the given form class.
    """
    fields = []
    for name, field in form_class.declared_fields.items():
        fields.append(
            {
                "name": name,
                "required": field.required,
                "in": "query",
                "description": force_str(field.help_text) if field.help_text else "",
                "schema": get_form_field_openapi_schema(field),
            }
        )
    return fields


def get_form_field_openapi_schema(field):
    if isinstance(field, forms.BooleanField):
        return {"type": "boolean"}
    elif isinstance(field, forms.DateField):
        return {"type": "string", "format": "date"}
    elif isinstance(field, forms.DateTimeField):
        return {"type": "string", "format": "date-time"}
    elif isinstance(field, forms.FloatField):
        return {"type": "number"}
    elif isinstance(field, (forms.IntegerField, forms.ModelChoiceField)):
        return {"type": "integer"}
    else:
        return {"type": "string"}



# -*- coding: utf-8 -*-

from django.test import TestCase

from rest_search.forms import SearchForm
from tests.forms import BookSearchForm


class FormsTest(TestCase):
    def test_base(self):
        form = SearchForm({})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.get_query(), {"match_all": {}})

    def test_empty(self):
        form = BookSearchForm({})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.get_query(), {"match_all": {}})

    def test_with_id(self):
        form = BookSearchForm({"id": 1})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.get_query(), {"bool": {"filter": [{"term": {"id": 1}}]}})

    def test_with_query(self):
        form = BookSearchForm({"query": "foo"})
        self.assertTrue(form.is_valid())
        self.assertEqual(
            form.get_query(),
            {
                "bool": {
                    "must": [
                        {"simple_query_string": {"fields": ["name"], "query": "foo"}}
                    ]
                }
            },
        )

    def test_with_tags(self):
        form = BookSearchForm({"tags": "tag1,tag2"})
        self.assertTrue(form.is_valid())
        self.assertEqual(
            form.get_query(),
            {
                "bool": {
                    "filter": [{"term": {"tags": "tag1"}}, {"term": {"tags": "tag2"}}]
                }
            },
        )

    def test_with_tags_and_query(self):
        form = BookSearchForm({"query": "foo", "tags": "tag1,tag2"})
        self.assertTrue(form.is_valid())
        self.assertEqual(
            form.get_query(),
            {
                "bool": {
                    "filter": [{"term": {"tags": "tag1"}}, {"term": {"tags": "tag2"}}],
                    "must": [
                        {"simple_query_string": {"fields": ["name"], "query": "foo"}}
                    ],
                }
            },
        )
